﻿using System;
using System.Reflection;
using System.Collections.Generic;

namespace Encompass
{
    public abstract class EntityRenderer : Renderer, IEntityTracker
    {
        private readonly List<Type> componentTypes = new List<Type>();
        private EntityTracker entityTracker = new EntityTracker();

        public IEnumerable<Type> ComponentTypes { get { return componentTypes; } }
        public Type DrawComponentType { get; }

        public abstract void Render(Entity entity);

        public EntityRenderer()
        {
            var rendersAttribute = GetType().GetCustomAttribute<Renders>(false);
            if (rendersAttribute != null)
            {
                componentTypes = rendersAttribute.componentTypes;
                DrawComponentType = rendersAttribute.drawComponentType;
            }
        }

        public bool CheckAndTrackEntity(Guid entityID)
        {
            var entity = GetEntity(entityID);
            var shouldTrack = CheckEntity(entity);
            if (shouldTrack) { entityTracker.TrackEntity(entityID); }
            return shouldTrack;
        }

        public bool CheckAndUntrackEntity(Guid entityID)
        {
            var entity = GetEntity(entityID);
            var shouldUntrack = CheckEntity(entity);
            if (shouldUntrack) { entityTracker.UntrackEntity(entityID); }
            return shouldUntrack;
        }

        public bool IsTracking(Guid entityID)
        {
            return entityTracker.IsTracking(entityID);
        }

        internal bool CheckEntity(Entity entity)
        {
            return EntityChecker.CheckEntity(entity, componentTypes) &&
                entity.HasComponent(DrawComponentType);
        }
    }
}
